/* File *******************************************************************
*
*					North Carolina State University
*					Electronics Research Laboratory
*
* FILENAME :    s2ibis.h
*
* DESCRIPTION : This file supplies all of the header info required by s2ibis.
*
*
* $Id: s2ibis.h,v 0.91 1995/09/27 16:42:53 awglaser Exp $
*
*
* $Log: s2ibis.h,v $
*   Revision 0.91  1995/09/27  16:42:53  awglaser
*   Added Id, Log and copyright info to file header.
*   Changed VERSION_STRING to "0.91BETA"
*
*
*
* Copyright 1994, North Carolina State University.
*
* North Carolina State University (NCSU) and the author hereby disclaim
* all implied warranties, including the implied warranties of merchant-
* ability and fitness for a particular purpose. NCSU is not liable for
* any damages incurred by the use of this software and/or documentation,
* including direct, indirect, special, incidental, or consequential
* damages.
*
* NCSU grants the bearer the right to modify, copy, and redistribute
* this software and documentation, but requests that NCSU be acknowledged
* for its contribution in the resulting product.
*
*H*/
#include <stdlib.h>


#define UNDEFINED						0
#define OUTPUT_LOW						0
#define OUTPUT_HIGH						1
#define OUTPUT_FALLING					0
#define OUTPUT_RISING					1
#define DISABLE_OUTPUT					0
#define	ENABLE_OUTPUT					1

#define VERSION_STRING					"0.91BETA"

#define NUM_PTS_LINEAR_REGION			10

/* ibis version 1.0 Restriction */
#define MAX_TABLE_SIZE					100
#define MAX_MANUFACTURERS_NAME_LENGTH	40
#define MAX_FILENAME_BASE_LENGTH		8
#define FILENAME_EXTENSION				"ibs"
#define FILENAME_EXTENSION_LENGTH		3
#define MAX_PIN_NAME_LENGTH				5
#define MAX_SIGNAL_NAME_LENGTH			20
#define MAX_MODEL_NAME_LENGTH			20
#define MAX_R_PIN_STRING_LENGTH			9
#define MAX_L_PIN_STRING_LENGTH			9
#define MAX_C_PIN_STRING_LENGTH			9
#define MAX_LINE_LENGTH					80
#define MAX_COMPONENT_NAME_LENGTH		40

/* additional ibis version 2.0 Restriction */
#define MAX_PACKAGE_MODEL_NAME_LENGTH	40
#define MAX_PIN_MAPPING_NAME_LENGTH		15
#define MAX_DFF_PIN_NAME_LENGTH			5
#define MAX_INV_PIN_LEBGTH				5
#define MAX_VDIFF_TYP_LENGTH			9
#define MAX_TEDELAY_MAX_LENGTH			9
#define MAX_WAVEFORM_TABLES				100

/* Local (assumed) Limits */

#define MAX_FILE_REV_LENGTH     		4
#define MAX_TOK_LEN						40	
#define MAX_DATE_STRING_LENGTH			40
#define MAX_IBIS_STRING_LENGTH			1024 
					/* Applies to all other strings such
				     * as [Notes], [Disclaimer] etc.
				     * The longer strings do not indicate an
				     * error in the input file but they are
				     * truncated to this length for internal
				     * storage.
				     */

#define IBIS_PRINT_WIDTH 				80


/* derived limits */
#define MAX_FILENAME_LENGTH \
                 MAX_FILENAME_BASE_LENGTH + 1 + FILENAME_EXTENSION_LENGTH


/* illegal comment characters in version 1.0 */
#define ONE_ZERO_ILLEGAL_COMMENT_CHARS	""

/* illegal comment characters in version 2.0 */
#define TWO_ZERO_ILLEGAL_COMMENT_CHARS \
         "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789[]._/+-"



#define USE_NA							(-1.33287736222333E20)
#define NOT_USED						(-1.33287736222333E18)
#define TOLERANCE						(1e15)

#define IS_USE_NA( number ) ( number >= USE_NA - TOLERANCE ) && \
						( number <= USE_NA + TOLERANCE )

#define IS_NOT_USED( number ) ( number >= NOT_USED - TOLERANCE ) && \
							  ( number <= NOT_USED + TOLERANCE )

#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define MAX(a,b) ((a) > (b) ? (a) : (b))

#define ROUND( num ) (int) ( num + 0.5 )


#define majorReleaseIncrement 1000 /* Limits number of minor releases to 20 */
#define minorReleaseIncrement 50   /* lmits number of Type specific IDs to 50 */

#define VERSION_ID( MajorRelease, MinorRelease, SpecificID) \
        ((MajorRelease * majorReleaseIncrement) \
           + (MinorRelease * minorReleaseIncrement) \
             +  SpecificID )


#define VERSION_ONE_ZERO				VERSION_ID( 1, 0,  0)
#define VERSION_ONE_ONE					VERSION_ID( 1, 1,  0)
#define VERSION_TWO_ZERO				VERSION_ID( 2, 0,  0)
#define VERSION_TWO_ONE					VERSION_ID( 2, 1,  0)

/* Model Types */
#define MODEL_TYPE_UNDEFINED			UNDEFINED
   /* version 1.0 */
#define MODEL_TYPE_INPUT				VERSION_ID( 1, 0,  1)
#define MODEL_TYPE_OUTPUT				VERSION_ID( 1, 0,  2)
#define MODEL_TYPE_3_STATE				VERSION_ID( 1, 0,  3)
#define MODEL_TYPE_OPEN_DRAIN			VERSION_ID( 1, 0,  4)
#define MODEL_TYPE_I_O					VERSION_ID( 1, 0,  5)
   /* version 2.0 */
#define MODEL_TYPE_I_O_OPEN_DRAIN		VERSION_ID( 2, 0,  1)
#define MODEL_TYPE_I_O_OPEN_SINK		VERSION_ID( 2, 0,  2)
#define MODEL_TYPE_I_O_OPEN_SOURCE		VERSION_ID( 2, 0,  3)
#define MODEL_TYPE_INPUT_ECL			VERSION_ID( 2, 0,  4)
#define MODEL_TYPE_I_O_ECL				VERSION_ID( 2, 0,  5)
#define MODEL_TYPE_TERMINATOR			VERSION_ID( 2, 0,  6)
#define MODEL_TYPE_OPEN_SINK			VERSION_ID( 2, 0,  7)
#define MODEL_TYPE_OPEN_SOURCE			VERSION_ID( 2, 0,  8)
#define MODEL_TYPE_OUTPUT_ECL			VERSION_ID( 2, 0,  9)


#define	IS_ECL( model )	( model == MODEL_TYPE_INPUT_ECL ) ||\
						( model == MODEL_TYPE_I_O_ECL ) ||\
						( model == MODEL_TYPE_OUTPUT_ECL )


/* Model Enable types */
#define MODEL_ENABLE_UNDEFINED			UNDEFINED
   /* version 1.0 */
#define MODEL_ENABLE_ACTIVE_HIGH		VERSION_ID( 1, 2,  1)
#define MODEL_ENABLE_ACTIVE_LOW			VERSION_ID( 1, 2,  2)

/* Model Polarity types */
#define MODEL_POLARITY_UNDEFINED		UNDEFINED
   /* version 1.0 */
#define MODEL_POLARITY_NON_INVERTING	VERSION_ID( 1, 3,  1)
#define MODEL_POLARITY_INVERTING		VERSION_ID( 1, 3,  2)


/* Supported Spice types */
#define SPICE_TYPE_HSPICE				0
#define SPICE_TYPE_PSPICE				1
#define SPICE_TYPE_SPICE2				2
#define SPICE_TYPE_SPICE3				3
#define SPICE_TYPE_SPECTRE				4


#define IS_AN_INTEGER_NODE_NAME( name, dummy ) \
						sscanf( name, "%d", &dummy )


/* VI Curve types */
#define CURVE_TYPE_PULLUP				0
#define CURVE_TYPE_PULLDOWN				1
#define CURVE_TYPE_POWER_CLAMP			2
#define CURVE_TYPE_GND_CLAMP			3
#define CURVE_TYPE_DISABLED_PULLUP		4
#define CURVE_TYPE_DISABLED_PULLDOWN	5
#define CURVE_TYPE_RISING_RAMP			6
#define CURVE_TYPE_FALLING_RAMP			7
#define CURVE_TYPE_RISING_WAVE			8
#define CURVE_TYPE_FALLING_WAVE			9


/*-------------------------------------------------------------------------
*
*	Now define some default values.  These will go into the global data 
*	structure, where they will be subsequently copied to the components and
*	then the models if they are not otherwise specified.
*
*------------------------------------------------------------------------*/
#define RLOAD_DEFAULT 					50
#define C_COMP_DEFAULT 					5E-12
#define	R_PKG_DEFAULT 					0
#define	L_PKG_DEFAULT 					0
#define	C_PKG_DEFAULT 					0
#define TEMP_TYP_DEFAULT 				50
#define TEMP_MIN_DEFAULT 				0
#define TEMP_MAX_DEFAULT 				100
#define VOLTAGE_RANGE_TYP_DEFAULT		5
#define VOLTAGE_RANGE_MIN_DEFAULT		4.5
#define VOLTAGE_RANGE_MAX_DEFAULT		5.5
#define SIM_TIME_DEFAULT				10e-9
#define VINL_ECL_DEFAULT				-1.475
#define VINH_ECL_DEFAULT				-1.165
#define VINL_STD_DEFAULT				0.8
#define VINH_STD_DEFAULT				2.0
#define SPICE_TYPE_DEFAULT				SPICE_TYPE_HSPICE
#define LIN_RANGE_DEFAULT				5.0
#define SWEEP_STEP_DEFAULT				0.1
#define DIODE_DROP_DEFAULT				1.0
#define ECL_SWEEP_RANGE_DEFAULT			2.2
#define DERATE_VI_PCT_DEFAULT			0
#define DERATE_RAMP_PCT_DEFAULT			0
#define	ECL_TERMINATION_VOLTAGE_DEFAULT	-2.0
#define WAVE_POINTS_DEFAULT				51

/*
 * ARGUMENT LABELS
 *
 * These are used to label function arguments.
 *
 * IN indicates that the argument data is read only by the function, and 
 * not written.
 *
 * INOUT indicates that the argument data is read and written by the
 * function.
 *
 * OUT indicates that the argument data is written by the function,
 * without reference to its previous value.
 *
 * INOUT and OUT apply only to pointer arguments, since there is no
 * direct pass-by-reference mechanism in C.
 *
 * A fourth keyword, SHARED, is also defined.  It indicates that the
 * argument is a pointer to shared data (i. e., shared between PROCESSES
 * or simultaneously-running coroutines).  Data can be read or written,
 * but not meaningfully copied, since the value of the data pointed to
 * may CHANGE at any time.  A good example would be an address containing
 * a time.
 *
 * Example:
 *
 * char CharFunc(   IN readOnlyArg,
 *		    		OUT writeOnlyArg, writeOnlyArg2,
 *		  			INOUT readWriteArg,
 *		 			SHARED theTime)
 */

#define	IN
#define OUT
#define INOUT
#define SHARED



#define SIGN(a,b) ( b >= 0 ? (a >= 0 ? a : - a) : (a >= 0 ? - a : a))
#define ABORT() fflush(stderr);fflush(stdout);abort();
#define MALLOC(x) calloc(1,(unsigned)(x))
#define FREE(x) {free((char *)(x));(x) = 0;}
#define REALLOC(x,y) realloc((char *)(x),(unsigned)(y))


#define INITIAL_LOAD_STRING( destination, source, max_length ) \
                     if((destination)[0] == 0) \
                       LOAD_STRING( destination, source, max_length )


#define LOAD_STRING( destination, source, max_length ) \
                     if(strlen(source) > max_length) \
                       yyerror("string too long"); \
				strncpy(destination, source, max_length);


typedef struct {
	double			typ,			/* typical value */
					min,			/* minimum value */
					max;			/* maximum value */
} ibisTypMinMax_t, *ibisTypMinMax_Pt;


typedef struct {
	ibisTypMinMax_t R_pkg,
			 		L_pkg,
			 		C_pkg;
} ibisPackage_t, *ibisPackage_Pt;



typedef struct {
	ibisTypMinMax_t dv_r,
					dt_r,
					dv_f,
					dt_f;
} ibisRamp_t, *ibisRamp_Pt;


typedef struct VIentryDef {
	double			v;
	ibisTypMinMax_t	i;
} ibisVItableEntry_t, *ibisVItableEntry_Pt;


typedef struct {
	int				size;
    ibisVItableEntry_Pt 
					VIs;
} ibisVItable_t, *ibisVItable_Pt;


typedef struct waveTableEntryDef {
	double			t;
	ibisTypMinMax_t	v;
} ibisWaveTableEntry_t, *ibisWaveTableEntry_Pt;

typedef struct waveTableDef waveTable;
typedef struct waveTableDef {
	double			R_fixture,		/* device physical... */
					V_fixture,		/* parms...see IBIS spec */
					V_fixture_min,
					V_fixture_max,
					L_fixture,
					C_fixture,
					R_dut,
					L_dut,
					C_dut;
	int				size;			/* table size -- max 100 */
	ibisWaveTableEntry_Pt
					waveData;	/* array pointer to data */
	waveTable		*nextWaveTable_P;	/* pointer to next table */
} ibisWaveTable_t, *ibisWaveTable_Pt;
    


typedef struct modelStructDef modelStruct;
typedef struct modelStructDef {
	char 			modelName[MAX_MODEL_NAME_LENGTH + 1],
					modelFileTyp[MAX_FILENAME_LENGTH + 1],
					modelFileMin[MAX_FILENAME_LENGTH + 1],
					modelFileMax[MAX_FILENAME_LENGTH + 1];
	int     		modelType,
	        		polarity,
	        		enable,
					hasBeenAnalyzed;	/* 0 if no, >=1 if yes */
	double  		Vinl,
					Vinh,
					Vmeas,
					Cref,
					Rref,
					Vref,
					Rload,
					derateRampPct,
					derateVIPct,
					simTime;
	ibisTypMinMax_t c_comp,
					tempRange,
					voltageRange,
					pullupRef,
					pulldownRef,
					powerClampRef,
					gndClampRef,
					Rgnd,
					Rpower,
					Rac,
					Cac;
	ibisVItable_Pt	pulldown,
					pullup,
					gnd_clamp,
					power_clamp;
	ibisWaveTable_Pt	
					risingWave_P,
					fallingWave_P;
	ibisRamp_t		ramp;
	modelStruct 	*nextModel_P;
} ibisModel_t, *ibisModel_Pt;



typedef struct shortPinStructDef shortPinStruct;
typedef struct shortPinStruct {
	char			spiceNodeName[MAX_PIN_NAME_LENGTH + 1];
	char			pinName[MAX_PIN_NAME_LENGTH + 1];
	shortPinStruct	*nextPin_P;
} ibisShortPin_t, *ibisShortPin_Pt;


typedef struct diffPinStructDef diffPinStruct;
typedef struct  diffPinStructDef {
	char			pinName[MAX_PIN_NAME_LENGTH + 1],
					invPinName[MAX_PIN_NAME_LENGTH + 1];
	double			vdiff,
					tdelayTyp,
					tdelayMin,
					tdelayMax;
	diffPinStruct	*nextDiffPin_P;
} ibisDiffPin_t, *ibisDiffPin_Pt;


typedef struct pinMapStructDef pinMapStruct;
typedef struct  pinMapStructDef {
	char			pinName[MAX_PIN_NAME_LENGTH + 1],
					pullupRef[MAX_PIN_MAPPING_NAME_LENGTH + 1],
					pulldownRef[MAX_PIN_MAPPING_NAME_LENGTH + 1],
					powerClampRef[MAX_PIN_MAPPING_NAME_LENGTH + 1],
					gndClampRef[MAX_PIN_MAPPING_NAME_LENGTH + 1];
	pinMapStruct 	*nextPinMap_P;	/* next pin map in the list */
} ibisPinMap_t, *ibisPinMap_Pt;


typedef struct pinStructDef pinStruct;
typedef struct  pinStructDef {
	char			spiceNodeName[MAX_PIN_NAME_LENGTH + 1],
		 			pinName[MAX_PIN_NAME_LENGTH + 1],
		 			signalName[MAX_SIGNAL_NAME_LENGTH + 1],
		 			modelName[MAX_MODEL_NAME_LENGTH + 1],
					inputPin[MAX_PIN_NAME_LENGTH + 1],
					enablePin[MAX_PIN_NAME_LENGTH + 1],
					pullupRef[MAX_PIN_MAPPING_NAME_LENGTH + 1],
					pulldownRef[MAX_PIN_MAPPING_NAME_LENGTH + 1],
					powerClampRef[MAX_PIN_MAPPING_NAME_LENGTH + 1],
					gndClampRef[MAX_PIN_MAPPING_NAME_LENGTH + 1];
	float			R_pin,
					L_pin,
					C_pin;
	ibisModel_Pt	model_P;	/* points to model for this pin */
	pinStruct 		*nextPin_P;	/* next pin in the list */
} ibisPin_t, *ibisPin_Pt;


typedef struct componentStructDef componentStruct;
typedef struct componentStructDef  {
	int				hasPinMapping;
	char			component[MAX_COMPONENT_NAME_LENGTH + 1],
					packageModel[MAX_PACKAGE_MODEL_NAME_LENGTH + 1],
					manufacturer[MAX_MANUFACTURERS_NAME_LENGTH + 1],
					spiceFile[MAX_FILENAME_LENGTH + 1];
	ibisTypMinMax_t	tempRange,
					voltageRange,
					pullupRef,
					pulldownRef,
					powerClampRef,
					gndClampRef,
					c_comp;
	double			Rload,
					derateVIPct,
					derateRampPct,
					simTime;
	ibisPackage_t	pinParasitics;		/* pin parasitics */
	ibisPin_Pt 		pins_P;				/* head of pin list */
	ibisShortPin_Pt inputs_P;			/* list of input pins */
	ibisModel_Pt	models_P;			/* head of models list */
	ibisPinMap_Pt	pinMap_P;			/* pin map list */
	ibisDiffPin_Pt	diffPin_P;
	componentStruct	*nextComponent_P;	/* next component */
} ibisComponent_t, *ibisComponent_Pt;


typedef struct  {
	int	    		ibisVersion,
					iterate,
					cleanup,
					spiceType;
	char			thisFileName[MAX_FILENAME_LENGTH + 1],
					fileRev[MAX_FILE_REV_LENGTH + 1],
					date[MAX_DATE_STRING_LENGTH + 1],
					*source,
					*notes,
					*disclaimer,
					*copyright,
					*spiceCommand;
	ibisComponent_Pt
					components_P;
} ibisTOP_t, *ibisTOP_Pt;


typedef struct	{
	char			commentChar;
	ibisTypMinMax_t	tempRange,
					voltageRange,
					pullupRef,
					pulldownRef,
					powerClampRef,
					gndClampRef,
					c_comp;
	double			Rload,
					derateVIPct,
					derateRampPct,
					simTime;
	ibisPackage_t	pinParasitics;
} ibisGlobal_t, *ibisGlobal_Pt;
